<?php

require_once('ApiSettings.php');
require_once('Domain.php');
require_once('Nameserver.php');
require_once('WhoisContact.php');
require_once('DnsEntry.php');
require_once('DomainBranding.php');

/**
 * This is the API endpoint for the DomainService
 *
 * @package Transip
 * @class DomainService
 * @author TransIP (support@transip.nl)
 * @version 20090811 08:04
 */
class Transip_DomainService
{
	// These fields are SOAP related
	const WSDL = 'https://api.transip.nl/wsdl/?service=DomainService';
	protected static $_soapClient = null;

	/**
	 * Gets the singleton SoapClient which is used to connect to the  TransIP Api
	 *
	 * @return SoapClient the SoapClient object to which we can connect to the TransIP API
	 */
	public static function _getSoapClient()
	{
		if(is_null(self::$_soapClient))
		{
			$classMap = array(
				'Domain' => 'Transip_Domain',
				'Nameserver' => 'Transip_Nameserver',
				'WhoisContact' => 'Transip_WhoisContact',
				'DnsEntry' => 'Transip_DnsEntry',
				'DomainBranding' => 'Transip_DomainBranding',
			);

			$options = array(
				'classmap'  => $classMap,
				'encoding' => 'utf-8', // lets support unicode
				'features' => SOAP_SINGLE_ELEMENT_ARRAYS, // see http://bugs.php.net/bug.php?id=43338
				'trace' => false, // can be used for debugging
			);

			self::$_soapClient = new SoapClient(self::WSDL, $options);

			self::$_soapClient->__setCookie('login', Transip_ApiSettings::$login);
			self::$_soapClient->__setCookie('hash', Transip_ApiSettings::$hash);
			self::$_soapClient->__setCookie('mode', Transip_ApiSettings::$mode);
		}

		return self::$_soapClient;
	}

	const AVAILABILITY_INYOURACCOUNT = 'inyouraccount';
	const AVAILABILITY_UNAVAILABLE = 'unavailable';
	const AVAILABILITY_NOTFREE = 'notfree';
	const AVAILABILITY_FREE = 'free';
	const REGISTRATIONSTATUS_GONE = 'gone';
	const CANCELLATIONTIME_END = 'end';
	const CANCELLATIONTIME_IMMEDIATELY = 'immediately';

	/**
	 * Checks the availability of a domain.
	 *
	 * @param string $domainName the domain name to check for availability
	 * @return string the availability status of the domain name:
	 * @example examples/checkAvailability.php
	 * @documentation documentation/checkAvailability.html
	 */
	public static function checkAvailability($domainName)
	{
		return self::_getSoapClient()->checkAvailability($domainName);
	}

	/**
	 * Gets the whois of a domain name
	 *
	 * @param string $domainName the domain name to get the whois for
	 * @return string the whois data for the domain
	 */
	public static function getWhois($domainName)
	{
		return self::_getSoapClient()->getWhois($domainName);
	}

	/**
	 * Gets the names of all domains in your account
	 *
	 * @return string[] A list of all domains in your account
	 */
	public static function getDomainNames()
	{
		return self::_getSoapClient()->getDomainNames();
	}

	/**
	 * Gets information about a domainName.
	 *
	 * @param string $domainName the domainName to get the information for
	 * @return Transip_Domain a Domain object holding the data for the requested domainName.
	 */
	public static function getInfo($domainName)
	{
		return self::_getSoapClient()->getInfo($domainName);
	}

	/**
	 * Gets the Auth code for a domainName
	 *
	 * @param string $domainName the domainName to get the authcode for
	 * @return string the authentication code for a domain name
	 */
	public static function getAuthCode($domainName)
	{
		return self::_getSoapClient()->getAuthCode($domainName);
	}

	/**
	 * Gets the lock status for a domainName
	 *
	 * @param string $domainName the domainName to get the lock status for
	 * @return boolean true iff the domain is locked at the registry
	 */
	public static function getIsLocked($domainName)
	{
		return self::_getSoapClient()->getIsLocked($domainName);
	}

	/**
	 * Registers a domain name, will automatically create and sign a proposition for it
	 *
	 * @param Transip_Domain $domain the Domain object holding information about the domain that needs to be registered.
	 * @requires readwrite mode
	 */
	public static function register($domain)
	{
		return self::_getSoapClient()->register($domain);
	}

	/**
	 * Cancels a domain name, will automatically create and sign a cancellation document
	 * Please note that domains with webhosting cannot be cancelled through the API
	 *
	 * @param string $domainName the domainname that needs to be cancelled
	 * @param string $endTime the time to cancel the domain (DomainService::CANCELLATIONTIME_END (end of contract) or DomainService::CANCELLATIONTIME_IMMEDIATELY (as soon as possible))
	 * @requires readwrite mode
	 */
	public static function cancel($domainName, $endTime)
	{
		return self::_getSoapClient()->cancel($domainName, $endTime);
	}

	/**
	 * Transfers a domain with changing the owner, not all TLDs support this (e.g. nl)
	 *
	 * @param Transip_Domain $domain the Domain object holding information about the domain that needs to be transfered
	 * @param string $authCode the authorization code for domains needing this for transfers (e.g. .com or .org transfers). Leave empty when n/a.
	 * @requires readwrite mode
	 */
	public static function transferWithOwnerChange($domain, $authCode)
	{
		return self::_getSoapClient()->transferWithOwnerChange($domain, $authCode);
	}

	/**
	 * Transfers a domain without changing the owner
	 *
	 * @param Transip_Domain $domain the Domain object holding information about the domain that needs to be transfered
	 * @param string $authCode the authorization code for domains needing this for transfers (e.g. .com or .org transfers). Leave empty when n/a.
	 * @requires readwrite mode
	 */
	public static function transferWithoutOwnerChange($domain, $authCode)
	{
		return self::_getSoapClient()->transferWithoutOwnerChange($domain, $authCode);
	}

	/**
	 * Starts a nameserver change for this domain, will replace all existing nameservers with the new nameservers
	 *
	 * @param string $domainName the domainName to change the nameservers for
	 * @param Transip_Nameserver[] $nameservers the list of new nameservers for this domain
	 */
	public static function setNameservers($domainName, $nameservers)
	{
		return self::_getSoapClient()->setNameservers($domainName, $nameservers);
	}

	/**
	 * Lock this domain in real time
	 *
	 * @param string $domainName the domainName to set the lock for
	 */
	public static function setLock($domainName)
	{
		return self::_getSoapClient()->setLock($domainName);
	}

	/**
	 * unlocks this domain in real time
	 *
	 * @param string $domainName the domainName to unlock
	 */
	public static function unsetLock($domainName)
	{
		return self::_getSoapClient()->unsetLock($domainName);
	}

	/**
	 * Sets the DnEntries for this Domain, will replace all existing dns entries with the new entries
	 *
	 * @param string $domainName the domainName to change the dns entries for
	 * @param Transip_DnsEntry[] $dnsEntries the list of new DnsEntries for this domain
	 */
	public static function setDnsEntries($domainName, $dnsEntries)
	{
		return self::_getSoapClient()->setDnsEntries($domainName, $dnsEntries);
	}

	/**
	 * Starts an owner change of a Domain, brings additional costs with the following TLDs:
	 * .nl
	 * .be
	 * .eu
	 *
	 * @param string $domainName the domainName to change the owner for
	 * @param Transip_WhoisContact $registrantWhoisContact the new contact data for this
	 */
	public static function setOwner($domainName, $registrantWhoisContact)
	{
		return self::_getSoapClient()->setOwner($domainName, $registrantWhoisContact);
	}

	/**
	 * Starts a contact change of a domain, this will replace all existing contacts
	 *
	 * @param string $domainName the domainName to change the contacts for
	 * @param Transip_WhoisContact[] $contacts the list of new contacts for this domain
	 */
	public static function setContacts($domainName, $contacts)
	{
		return self::_getSoapClient()->setContacts($domainName, $contacts);
	}
}

?>
